// Autor: Ingmar Stapel
// Homepage: https://custom-build-robots.com/
// Date: 2020-04-19
// This program was written to display on an OLED display 
// the received GPS information delivered by a UBLOX NEO 6 M module
// This program uses know how from the following sources:
// Source: https://iotdesignpro.com/projects/nodemcu-esp8266-gps-module-interfacing-to-display-latitude-and-longitude
#include <TinyGPS++.h> // library for GPS module
#include <SoftwareSerial.h>

// Only needed for Arduino 1.6.5 and earlier
#include <Wire.h>  
// load the library for the OLED display
#include "SSD1306Wire.h"
// use the default I2C address for the OLED display
SSD1306Wire  display(0x3c, D2, D1);

// The TinyGPS++ object
TinyGPSPlus gps;  
// The serial connection to the GPS device
SoftwareSerial ss(14, 12, false, 128);

// definition of the variables
float latitude , longitude;
int year , month , date, hour , minute , second;
String date_str , time_str , lat_str , lng_str;
int pm;
// set your local time zone hour
int time_zone_h = 2;
// set your local time zone minute
int time_zone_m = 0;
 
void setup()
{
  Serial.begin(115200);
  ss.begin(9600);

  Serial.println("Initializing OLED Display");
  display.init();

  display.flipScreenVertically();
  display.setFont(ArialMT_Plain_10);
}

// function to draw the information on the OLED display
void drawOLED(String time_str, String late, String longe) {
  
  // First clear the display
  display.clear();
  // set the position of the text
  display.setTextAlignment(TEXT_ALIGN_LEFT);
  
  // define the font and size
  display.setFont(ArialMT_Plain_16);
  // Draw the first line on the OLED display
  display.drawString(0, 0, "GPS - INFO");
  
  display.setFont(ArialMT_Plain_16);
  // draw the second line
  display.drawString(0, 16, "Late: " + String(late));

  display.setFont(ArialMT_Plain_16);
  // draw the third line
  display.drawString(0, 32, "Long: " + String(longe));

  display.setFont(ArialMT_Plain_16);
  // draw the fourth line
  display.drawString(0, 48, String(time_str));
  
  // write everything on the display
  display.display();
}

void loop()
{
  // set initial values for lat_str and lng_str
  lat_str = "-";
  lng_str = "-";
  
  //while data is available
  while (ss.available() > 0) 
    //read gps data
    if (gps.encode(ss.read())) 
    {
      //check whether gps location is valid
      if (gps.location.isValid()) 
      {
        // read the latitude value
        latitude = gps.location.lat();
        // latitude location is stored in a string
        lat_str = String(latitude , 6); 
        // read the longitude value
        longitude = gps.location.lng();
        //longitude location is stored in a string
        lng_str = String(longitude , 6); 
      }
      //check whether gps date is valid and get the time out of the data
      if (gps.date.isValid()) 
      {
        date_str = "";
        date = gps.date.day();
        month = gps.date.month();
        year = gps.date.year();
        if (date < 10)
          date_str = '0';
        // values of date,month and year are stored in a string
        date_str += String(date);
        date_str += " / ";

        if (month < 10)
          date_str += '0';
        // values of date,month and year are stored in a string
        date_str += String(month); 
        date_str += " / ";
        if (year < 10)
          date_str += '0';
        // values of date,month and year are stored in a string
        date_str += String(year); 
      }
      //check whether gps time is valid
      if (gps.time.isValid()) 
      {
        time_str = "";
        hour = gps.time.hour();
        minute = gps.time.minute();
        second = gps.time.second();
        // converting to IST
        minute = (minute + time_zone_m); 
        if (minute > 59)
        {
          minute = minute - 60;
          hour = hour + 1;
        }
        hour = (hour + time_zone_h) ;
        // converting to IST
        if (hour > 23)
          hour = hour - 24;   
        // checking whether AM or PM
        if (hour >= 12)  
          pm = 1;
        else
          pm = 0;
        hour = hour % 12;
        if (hour < 10)
          time_str = '0';
        //values of hour,minute and time are stored in a string
        time_str += String(hour); 
        time_str += " : ";
        if (minute < 10)
          time_str += '0';
        //values of hour,minute and time are stored in a string
        time_str += String(minute); 
        time_str += " : ";
        if (second < 10)
          time_str += '0';
        //values of hour,minute and time are stored in a string
        time_str += String(second); 
        if (pm == 1)
          time_str += " PM ";
        else
          time_str += " AM ";
      }
      drawOLED(time_str, lat_str, lng_str);
    }
  delay(100);
}
